@@ -4,7 +4,7 @@
  *  \version 1
  *
  *  \date Created: 04/12/04
- *  \date Last modified: Time-stamp: <2013-06-08 22:47:18 antoine>
+ *  \date Last modified: Time-stamp: <2023-02-03 12:04:13 (antoine)>
  *
  *  \author Antoine Lucas (help from Immanuel Scholz) (R adaptation)
  *          Original C code from libgmp.
@@ -12,6 +12,9 @@
  *  \note Licence: GPL (>= 2)
  */
 
+
+#include <stdexcept>
+
 #include "Rgmp.h"
 #include "factorize.h"
 using namespace std;
@@ -25,20 +28,23 @@ using namespace std;
 //
 SEXP factorR (SEXP n)
 {
-  bigvec v = bigintegerR::create_bignum(n), result;
-  if(v.size() > 0) {
+  try{
+    bigvec v = bigintegerR::create_bignum(n), result;
+    if(v.size() > 0) {
     mpz_t val;
     mpz_init(val);
     mpz_t_sentry val_s(val);
     mpz_set(val,v[0].getValue().getValueTemp());
 
     int sgn = mpz_sgn(val);
-    if(sgn == 0)
-      error(_("Cannot factorize 0"));
+    if(sgn == 0){
+      v.clear();
+      throw invalid_argument(_("Cannot factorize 0"));
+    }
     if(sgn<0)
       {
 	mpz_abs(val,val);
-	result.value.push_back(biginteger(-1));
+	result.push_back(bigmod(biginteger(-1)));
       }
     //
     // function from gmplib, in demo/factorize.c
@@ -46,4 +52,7 @@ SEXP factorR (SEXP n)
     factor(val,result);
   }
   return bigintegerR::create_SEXP(result);
+  }    catch(std::invalid_argument & e){
+    error(e.what());
+  }
 }
